# import os
import time
import calendar
from datetime import datetime

from ut_arr.aoeq import AoEq
from ut_log.log import Log

from typing import Any
TyAny = Any
TyDateTime = datetime
TyTimeStamp = int
TyArr = list[Any]
TyBool = bool
TyDic = dict[Any, Any]

TnAny = None | Any
TnArr = None | TyArr
TnDic = None | TyDic
TnTimeStamp = None | TyTimeStamp
TnDateTime = None | TyDateTime
TnStr = None | str


class Com:
    """
    Communication Class
    """
    cmd: TnStr = None
    sw_init: bool = False

    com_a_mod = None
    com_pac = None
    app_a_mod = None
    app_pac = None
    tenant: TnStr = None

    ts: TnTimeStamp
    d_timer: TyDic = {}

    Log: Any = None
    cfg: TnDic = None
    # App: Any = None
    # Exit: Any = None

    @classmethod
    def init(cls, kwargs: TyDic):
        # def init(cls, cls_app, kwargs: TyDic):
        """
        initialise static variables of Com class
        """
        if cls.sw_init:
            return
        cls.sw_init = True
        cls.cmd = kwargs.get('cmd')
        cls.com_a_mod = cls.__module__.split(".")
        cls.com_pac = cls.com_a_mod[0]
        _cls_app = kwargs.get('cls_app')
        cls.app_a_mod = _cls_app.__module__.split(".")
        cls.app_pac = cls.app_a_mod[0]
        cls.tenant = kwargs.get('tenant')
        cls.ts = calendar.timegm(time.gmtime())

        cls.Log = Log.sh(**kwargs)
        # cls.Cfg = Cfg.sh(cls, **kwargs)
        # cls.App = App.sh(cls, **kwargs)
        # cls.Exit = Exit.sh(**kwargs)

    @classmethod
    def sh_kwargs(cls, cls_app, cls_parms, *args) -> TyDic:
        """
        show keyword arguments
        """
        _kwargs: TyDic = AoEq.sh_d_eq(*args, d_parms=cls_parms.d_eq)
        _kwargs['cls_app'] = cls_app
        _kwargs['com'] = cls

        cls.init(_kwargs)
        return _kwargs



"""
Decorators Module
"""
import os
import sys
import numpy as np
from datetime import datetime

from ut_pac.fnc import Fnc
from ut_log.log import Log


def timer(fnc):
    """
    Timer Decorator
    """
    def wrapper(*args, **kwargs):
        start = datetime.now()
        fnc(*args, **kwargs)
        _fnc_name = Fnc.sh_fnc_name(fnc)
        end = datetime.now()
        elapse_time = end.timestamp() - start.timestamp()
        np_elapse_time = np.format_float_positional(elapse_time, trim='k')
        msg = f"{_fnc_name} elapse time [sec] = {np_elapse_time}"
        # Log.info(msg, stacklevel=2)
        Log.info(msg)
    return wrapper

def timer_ptc(fnc):
    """
    Timer Decorator
    """
    def wrapper(*args, **kwargs):
        start = datetime.now()
        fnc(*args, **kwargs)
        _fnc_name = Fnc.sh_fnc_name(fnc)
        end = datetime.now()
        elapse_time = end.timestamp() - start.timestamp()
        np_elapse_time = np.format_float_positional(elapse_time, trim='k')

        _d_run = kwargs.get("d_run", {})
        _run_method: str = _d_run.get("method", "sp")

        msg = f"{_fnc_name} _run_method = {_run_method} elapse time [sec] = {np_elapse_time}"
        # Log.info(msg, stacklevel=2)
        Log.info(msg)
    return wrapper

def handle_exception(exc_type, exc_value, exc_traceback):
    if issubclass(exc_type, KeyboardInterrupt):
        sys.__excepthook__(exc_type, exc_value, exc_traceback)
        return
    Log.critical(exc_value, exc_info=(exc_type, exc_value, exc_traceback))

def handle_error(fnc):
    """
    Error Decorator
    """
    def dec_handle_error(*args, **kwargs):
        try:
            fnc(*args, **kwargs)
            os._exit(0)
        except Exception:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            if issubclass(exc_type, KeyboardInterrupt):
                sys.__excepthook__(exc_type, exc_value, exc_traceback)
            else:
                Log.critical(exc_value, exc_info=(exc_type, exc_value, exc_traceback))
                os._exit(99)
    return dec_handle_error



# coding=utf-8
from datetime import datetime

from ut_com.com import Com
from ut_log.log import Log
from ut_pac.fnc import Fnc

from typing import Any
TyAny = Any
TyArr = list[Any]
TyDic = dict[Any, Any]
TyStr = str
TnAny = None | TyAny
TnStr = None | TyStr


class Timestamp:

    @staticmethod
    def sh_elapse_time_sec(end: Any, start: TnAny) -> TnAny:
        if start is None:
            return None
        return end.timestamp()-start.timestamp()


class Timer:
    """ Timer Management
    """
    @staticmethod
    def sh_args_str(*args) -> TyStr:
        """
        Show class name, the item class_name is the class_id if its a string,
        otherwise the attribute __qualname__ is used.
        """
        if not args:
            return ""
        else:
            return f"{args}"

    @staticmethod
    def sh_kwargs_str(**kwargs) -> TyStr:
        """
        Show class name, the item class_name is the class_id if its a string,
        otherwise the attribute __qualname__ is used.
        """
        if not kwargs:
            return ""
        else:
            return f"{kwargs}"

    @classmethod
    def sh_task_id(cls, fnc, *args, **kwargs) -> TyStr:
        """
        Show task id, which is created by the concationation of the following items:
        package, module, class_name and parms if they are defined; the items package
        and module are get from the package-module directory; the item class_name is
        the class_id if its a string, otherwise the attribute __qualname__ is used.
        """
        _pac_name = Fnc.sh_pac_name(fnc)
        _mod_name = Fnc.sh_mod_name(fnc)
        _cls_name = Fnc.sh_cls_name(fnc)
        _args = cls.sh_args_str(*args)
        _kwargs = cls.sh_kwargs_str(**kwargs)
        _sep = kwargs.get('sep', '.')
        task_id: TyStr = _sep.join([_pac_name, _mod_name, _cls_name, _args, _kwargs])
        return task_id

    @classmethod
    def start(cls, fnc: TyAny, *args, **kwargs) -> None:
        """
        Start Timer
        """
        task_id = cls.sh_task_id(fnc, *args, **kwargs)
        Com.d_timer[task_id] = datetime.now()

    @classmethod
    def end(cls, fnc: TyAny, *args, **kwargs) -> None:
        """
        End Timer
        """
        task_id = cls.sh_task_id(fnc, *args, **kwargs)
        start = Com.d_timer.get(task_id)
        end = datetime.now()
        elapse_time_sec = Timestamp.sh_elapse_time_sec(end, start)
        msg = f"{task_id} elapse time [sec] = {elapse_time_sec}"
        Log.info(msg, stacklevel=2)
